/* 
    Zwei 800x600 Bilder mit 8b je Farbkomponente (Rot, Grn, Blau)
    im XV PPM-Format berblenden (mischen) und das Ergebnis
    in eine weitere PPM-Datei schreiben. 

    Der Mischfaktor alpha gibt dabei an, ob das Ausgabebild nur aus den
    Pixeln des ersten Bildes (alpha=255), nur denen des 2. Bildes
    (alpha=0), oder einer gewichteten Summe bestehen soll (z.B. bei
    alpha=128 gehen die beiden Pixel hlftig ein.)
    
   Aufruf:

      ./blending image1.ppm image2.ppm 255 foo.ppm

      image1.ppm wird voll gewichtet in foo.ppm geschrieben

      ./blending image1.ppm image2.ppm 128 foo.ppm

      image1.ppm und image2.ppm werden berblendet in foo.ppm geschrieben
*/

#include <stdio.h>
   
#define HEIGHT 600
#define WIDTH  800
#define NUMPIXELS (HEIGHT*WIDTH)
#define COMPONENTS (NUMPIXELS*3)  // 3 farbkanle (rot,grn,blau) je Pixel
#define IMGSIZE (COMPONENTS*sizeof(PIXEL))
typedef unsigned char PIXEL, ALPHA;


/* =======Hier geht der spannende Teil los========================== */

/* Bilder berblenden */

PIXEL *blend(PIXEL image1[], PIXEL image2[], ALPHA alpha) {

  // Speicher fr Ausgabebild anfordern
  PIXEL *blended = (PIXEL*) malloc(IMGSIZE);
  int n;
  
  // Falls Speicher bekommen, mischen ...
  if (blended) {

    // Ganzes Bild durchlaufen (die RGB-Komponenten aller Pixel)

    for (n=0; n<IMGSIZE; ++n) {
      // grerer zahlenbereich (fixpunkt, alle Werte *255)
      unsigned short temp;

      // Bildpunkte mischen
      temp = alpha * image1[n] + (255-alpha) * image2[n];

      // Jetzt Komponente wieder rckwandeln in bereich 0..255
      blended[n] = temp >> 8;
    } 


  }

  return blended;
}

/* =====Ab hier wird es wieder langweilig=========================== */

/* einlesen einer passenden ppm datei */

PIXEL *readimage(char *filename) {
  FILE *infile = fopen(filename, "r");
  PIXEL *image = (PIXEL*) malloc(IMGSIZE);
  char buf[256];

  if (infile && image) {
    fgets(buf, 80, infile);   // skip P6
    fgets(buf, 80, infile);   // skip # CREATOR
    fgets(buf, 80, infile);   // skip dimensions
    fgets(buf, 80, infile);   // skip depth
    fread(image, sizeof(PIXEL), COMPONENTS, infile);
    fclose(infile);
    return(image);
  }
  if (infile)
    fclose(infile);
}

/* schreiben als ppm datei */

void writeimage(char *filename, PIXEL *image) {
  FILE *outfile = fopen(filename, "w");

  fputs("P6\n", outfile);
  fputs("# CREATOR kludge\n", outfile);
  fputs("800 600\n", outfile);
  fputs("255\n", outfile);
  fwrite(image, sizeof(PIXEL), COMPONENTS, outfile);
  fclose(outfile);
}

/* hauptprogramm */

int main(int argc, char **argv) {
  PIXEL *image1 = readimage(argv[1]);
  PIXEL *image2 = readimage(argv[2]);
  ALPHA alpha = atoi(argv[3]);

  PIXEL *blended = blend(image1, image2, alpha);

  writeimage(argv[4], blended);

  free(image1);
  free(image2);
  free(blended);  
}


